@@ -1,6 +1,7 @@
 from contextlib import suppress
 
 from adaptive.learner.average_learner import AverageLearner
+from adaptive.learner.average_learner1D import AverageLearner1D
 from adaptive.learner.balancing_learner import BalancingLearner
 from adaptive.learner.base_learner import BaseLearner
 from adaptive.learner.data_saver import DataSaver, make_datasaver
@@ -20,6 +21,7 @@ __all__ = [
     "Learner1D",
     "Learner2D",
     "LearnerND",
+    "AverageLearner1D",
     "SequenceLearner",
 ]
 
