@@ -1,18 +1,19 @@
 import abc
 from contextlib import suppress
-from copy import deepcopy
+
+import cloudpickle
 
 from adaptive.utils import _RequireAttrsABCMeta, load, save
 
 
-def uses_nth_neighbors(n):
+def uses_nth_neighbors(n: int):
     """Decorator to specify how many neighboring intervals the loss function uses.
 
     Wraps loss functions to indicate that they expect intervals together
     with ``n`` nearest neighbors
 
     The loss function will then receive the data of the N nearest neighbors
-    (``nth_neighbors``) aling with the data of the interval itself in a dict.
+    (``nth_neighbors``) along with the data of the interval itself in a dict.
     The `~adaptive.Learner1D` will also make sure that the loss is updated
     whenever one of the ``nth_neighbors`` changes.
 
@@ -191,7 +192,7 @@ class BaseLearner(metaclass=_RequireAttrsABCMeta):
             self._set_data(data)
 
     def __getstate__(self):
-        return deepcopy(self.__dict__)
+        return cloudpickle.dumps(self.__dict__)
 
     def __setstate__(self, state):
-        self.__dict__ = state
+        self.__dict__ = cloudpickle.loads(state)
