@@ -1,22 +1,54 @@
+import collections.abc
 import itertools
 import math
-from collections.abc import Iterable
-from copy import deepcopy
+from copy import copy, deepcopy
+from typing import Any, Callable, Dict, List, Optional, Sequence, Set, Tuple, Union
 
 import cloudpickle
 import numpy as np
-import sortedcollections
-import sortedcontainers
+from sortedcollections.recipes import ItemSortedDict
+from sortedcontainers.sorteddict import SortedDict
 
 from adaptive.learner.base_learner import BaseLearner, uses_nth_neighbors
 from adaptive.learner.learnerND import volume
 from adaptive.learner.triangulation import simplex_volume_in_embedding
 from adaptive.notebook_integration import ensure_holoviews
+from adaptive.types import Float, Int, Real
 from adaptive.utils import cache_latest
 
+# -- types --
+
+# Commonly used types
+Interval = Union[Tuple[float, float], Tuple[float, float, int]]
+NeighborsType = Dict[float, List[Union[float, None]]]
+
+# Types for loss_per_interval functions
+NoneFloat = Union[Float, None]
+NoneArray = Union[np.ndarray, None]
+XsType0 = Tuple[Float, Float]
+YsType0 = Union[Tuple[Float, Float], Tuple[np.ndarray, np.ndarray]]
+XsType1 = Tuple[NoneFloat, NoneFloat, NoneFloat, NoneFloat]
+YsType1 = Union[
+    Tuple[NoneFloat, NoneFloat, NoneFloat, NoneFloat],
+    Tuple[NoneArray, NoneArray, NoneArray, NoneArray],
+]
+XsTypeN = Tuple[NoneFloat, ...]
+YsTypeN = Union[Tuple[NoneFloat, ...], Tuple[NoneArray, ...]]
+
+
+__all__ = [
+    "uniform_loss",
+    "default_loss",
+    "abs_min_log_loss",
+    "triangle_loss",
+    "resolution_loss_function",
+    "curvature_loss_function",
+    "Learner1D",
+]
+
 
 @uses_nth_neighbors(0)
-def uniform_loss(xs, ys):
+def uniform_loss(xs: XsType0, ys: YsType0) -> Float:
     """Loss function that samples the domain uniformly.
 
     Works with `~adaptive.Learner1D` only.
@@ -36,7 +68,7 @@ def uniform_loss(xs, ys):
 
 
 @uses_nth_neighbors(0)
-def default_loss(xs, ys):
+def default_loss(xs: XsType0, ys: YsType0) -> Float:
     """Calculate loss on a single interval.
 
     Currently returns the rescaled length of the interval. If one of the
@@ -44,23 +76,24 @@ def default_loss(xs, ys):
     never touched. This behavior should be improved later.
     """
     dx = xs[1] - xs[0]
-    if isinstance(ys[0], Iterable):
-        dy = [abs(a - b) for a, b in zip(*ys)]
-        return np.hypot(dx, dy).max()
+    if isinstance(ys[0], collections.abc.Iterable):
+        dy_vec = np.array([abs(a - b) for a, b in zip(*ys)])
+        return np.hypot(dx, dy_vec).max()
     else:
         dy = ys[1] - ys[0]
         return np.hypot(dx, dy)
 
 
 @uses_nth_neighbors(0)
-def abs_min_log_loss(xs, ys):
+def abs_min_log_loss(xs: XsType0, ys: YsType0) -> Float:
     """Calculate loss of a single interval that prioritizes the absolute minimum."""
-    ys = [np.log(np.abs(y).min()) for y in ys]
+    ys = tuple(np.log(np.abs(y).min()) for y in ys)
     return default_loss(xs, ys)
 
 
 @uses_nth_neighbors(1)
-def triangle_loss(xs, ys):
+def triangle_loss(xs: XsType1, ys: YsType1) -> Float:
+    assert len(xs) == 4
     xs = [x for x in xs if x is not None]
     ys = [y for y in ys if y is not None]
 
@@ -68,7 +101,7 @@ def triangle_loss(xs, ys):
         return xs[1] - xs[0]
 
     N = len(xs) - 2  # number of constructed triangles
-    if isinstance(ys[0], Iterable):
+    if isinstance(ys[0], collections.abc.Iterable):
         pts = [(x, *y) for x, y in zip(xs, ys)]
         vol = simplex_volume_in_embedding
     else:
@@ -77,7 +110,9 @@ def triangle_loss(xs, ys):
     return sum(vol(pts[i : i + 3]) for i in range(N)) / N
 
 
-def resolution_loss_function(min_length=0, max_length=1):
+def resolution_loss_function(
+    min_length: Real = 0, max_length: Real = 1
+) -> Callable[[XsType0, YsType0], Float]:
     """Loss function that is similar to the `default_loss` function, but you
     can set the maximum and minimum size of an interval.
 
@@ -96,11 +131,11 @@ def resolution_loss_function(min_length=0, max_length=1):
     ...     return x**2
     >>>
     >>> loss = resolution_loss_function(min_length=0.01, max_length=1)
-    >>> learner = adaptive.Learner1D(f, bounds=[(-1, -1), (1, 1)], loss_per_triangle=loss)
+    >>> learner = adaptive.Learner1D(f, bounds=(-1, -1), loss_per_interval=loss)
     """
 
     @uses_nth_neighbors(0)
-    def resolution_loss(xs, ys):
+    def resolution_loss(xs: XsType0, ys: YsType0) -> Float:
         loss = uniform_loss(xs, ys)
         if loss < min_length:
             # Return zero such that this interval won't be chosen again
@@ -114,10 +149,12 @@ def resolution_loss_function(min_length=0, max_length=1):
     return resolution_loss
 
 
-def curvature_loss_function(area_factor=1, euclid_factor=0.02, horizontal_factor=0.02):
+def curvature_loss_function(
+    area_factor: Real = 1, euclid_factor: Real = 0.02, horizontal_factor: Real = 0.02
+) -> Callable[[XsType1, YsType1], Float]:
     # XXX: add a doc-string
     @uses_nth_neighbors(1)
-    def curvature_loss(xs, ys):
+    def curvature_loss(xs: XsType1, ys: YsType1) -> Float:
         xs_middle = xs[1:3]
         ys_middle = ys[1:3]
 
@@ -133,7 +170,7 @@ def curvature_loss_function(area_factor=1, euclid_factor=0.02, horizontal_factor
     return curvature_loss
 
 
-def linspace(x_left, x_right, n):
+def linspace(x_left: Real, x_right: Real, n: Int) -> List[Float]:
     """This is equivalent to
     'np.linspace(x_left, x_right, n, endpoint=False)[1:]',
     but it is 15-30 times faster for small 'n'."""
@@ -145,17 +182,19 @@ def linspace(x_left, x_right, n):
         return [x_left + step * i for i in range(1, n)]
 
 
-def _get_neighbors_from_list(xs):
+def _get_neighbors_from_array(xs: np.ndarray) -> NeighborsType:
     xs = np.sort(xs)
     xs_left = np.roll(xs, 1).tolist()
     xs_right = np.roll(xs, -1).tolist()
     xs_left[0] = None
     xs_right[-1] = None
     neighbors = {x: [x_L, x_R] for x, x_L, x_R in zip(xs, xs_left, xs_right)}
-    return sortedcontainers.SortedDict(neighbors)
+    return SortedDict(neighbors)
 
 
-def _get_intervals(x, neighbors, nth_neighbors):
+def _get_intervals(
+    x: float, neighbors: NeighborsType, nth_neighbors: int
+) -> List[Tuple[float, float]]:
     nn = nth_neighbors
     i = neighbors.index(x)
     start = max(0, i - nn - 1)
@@ -171,7 +210,7 @@ class Learner1D(BaseLearner):
     ----------
     function : callable
         The function to learn. Must take a single real parameter and
-        return a real number.
+        return a real number or 1D array.
     bounds : pair of reals
         The bounds of the interval on which to learn 'function'.
     loss_per_interval: callable, optional
@@ -208,8 +247,13 @@ class Learner1D(BaseLearner):
     decorator for more information.
     """
 
-    def __init__(self, function, bounds, loss_per_interval=None):
-        self.function = function
+    def __init__(
+        self,
+        function: Callable[[Real], Union[Float, np.ndarray]],
+        bounds: Tuple[Real, Real],
+        loss_per_interval: Optional[Callable[[XsTypeN, YsTypeN], Float]] = None,
+    ):
+        self.function = function  # type: ignore
 
         if hasattr(loss_per_interval, "nth_neighbors"):
             self.nth_neighbors = loss_per_interval.nth_neighbors
@@ -223,13 +267,13 @@ class Learner1D(BaseLearner):
         # the learners behavior in the tests.
         self._recompute_losses_factor = 2
 
-        self.data = {}
-        self.pending_points = set()
+        self.data: Dict[Real, Real] = {}
+        self.pending_points: Set[Real] = set()
 
         # A dict {x_n: [x_{n-1}, x_{n+1}]} for quick checking of local
         # properties.
-        self.neighbors = sortedcontainers.SortedDict()
-        self.neighbors_combined = sortedcontainers.SortedDict()
+        self.neighbors: NeighborsType = SortedDict()
+        self.neighbors_combined: NeighborsType = SortedDict()
 
         # Bounding box [[minx, maxx], [miny, maxy]].
         self._bbox = [list(bounds), [np.inf, -np.inf]]
@@ -246,11 +290,12 @@ class Learner1D(BaseLearner):
         self._dx_eps = 2 * max(np.abs(bounds)) * np.finfo(float).eps
 
         self.bounds = list(bounds)
+        self.__missing_bounds = set(self.bounds)  # cache of missing bounds
 
-        self._vdim = None
+        self._vdim: Optional[int] = None
 
     @property
-    def vdim(self):
+    def vdim(self) -> int:
         """Length of the output of ``learner.function``.
         If the output is unsized (when it's a scalar)
         then `vdim = 1`.
@@ -269,36 +314,45 @@ class Learner1D(BaseLearner):
                 return 1
         return self._vdim
 
+    def to_numpy(self):
+        """Data as NumPy array of size ``(npoints, 2)`` if ``learner.function`` returns a scalar
+        and ``(npoints, 1+vdim)`` if ``learner.function`` returns a vector of length ``vdim``."""
+        return np.array([(x, *np.atleast_1d(y)) for x, y in sorted(self.data.items())])
+
     @property
-    def npoints(self):
+    def npoints(self) -> int:
         """Number of evaluated points."""
         return len(self.data)
 
     @cache_latest
-    def loss(self, real=True):
+    def loss(self, real: bool = True) -> float:
+        if self._missing_bounds():
+            return np.inf
         losses = self.losses if real else self.losses_combined
         if not losses:
             return np.inf
         max_interval, max_loss = losses.peekitem(0)
         return max_loss
 
-    def _scale_x(self, x):
+    def _scale_x(self, x: Optional[Float]) -> Optional[Float]:
         if x is None:
             return None
         return x / self._scale[0]
 
-    def _scale_y(self, y):
+    def _scale_y(
+        self, y: Union[Float, np.ndarray, None]
+    ) -> Union[Float, np.ndarray, None]:
         if y is None:
             return None
         y_scale = self._scale[1] or 1
         return y / y_scale
 
-    def _get_point_by_index(self, ind):
+    def _get_point_by_index(self, ind: int) -> Optional[float]:
         if ind < 0 or ind >= len(self.neighbors):
             return None
         return self.neighbors.keys()[ind]
 
-    def _get_loss_in_interval(self, x_left, x_right):
+    def _get_loss_in_interval(self, x_left: float, x_right: float) -> float:
         assert x_left is not None and x_right is not None
 
         if x_right - x_left < self._dx_eps:
@@ -318,7 +372,9 @@ class Learner1D(BaseLearner):
         # we need to compute the loss for this interval
         return self.loss_per_interval(xs_scaled, ys_scaled)
 
-    def _update_interpolated_loss_in_interval(self, x_left, x_right):
+    def _update_interpolated_loss_in_interval(
+        self, x_left: float, x_right: float
+    ) -> None:
         if x_left is None or x_right is None:
             return
 
@@ -334,7 +390,7 @@ class Learner1D(BaseLearner):
             self.losses_combined[a, b] = (b - a) * loss / dx
             a = b
 
-    def _update_losses(self, x, real=True):
+    def _update_losses(self, x: float, real: bool = True) -> None:
         """Update all losses that depend on x"""
         # When we add a new point x, we should update the losses
         # (x_left, x_right) are the "real" neighbors of 'x'.
@@ -377,7 +433,7 @@ class Learner1D(BaseLearner):
             self.losses_combined[x, b] = float("inf")
 
     @staticmethod
-    def _find_neighbors(x, neighbors):
+    def _find_neighbors(x: float, neighbors: NeighborsType) -> Any:
         if x in neighbors:
             return neighbors[x]
         pos = neighbors.bisect_left(x)
@@ -386,14 +442,14 @@ class Learner1D(BaseLearner):
         x_right = keys[pos] if pos != len(neighbors) else None
         return x_left, x_right
 
-    def _update_neighbors(self, x, neighbors):
+    def _update_neighbors(self, x: float, neighbors: NeighborsType) -> None:
         if x not in neighbors:  # The point is new
             x_left, x_right = self._find_neighbors(x, neighbors)
             neighbors[x] = [x_left, x_right]
             neighbors.get(x_left, [None, None])[1] = x
             neighbors.get(x_right, [None, None])[0] = x
 
-    def _update_scale(self, x, y):
+    def _update_scale(self, x: float, y: Union[Float, np.ndarray]) -> None:
         """Update the scale with which the x and y-values are scaled.
 
         For a learner where the function returns a single scalar the scale
@@ -408,8 +464,8 @@ class Learner1D(BaseLearner):
         if y is not None:
             if self.vdim > 1:
                 try:
-                    y_min = np.min([self._bbox[1][0], y], axis=0)
-                    y_max = np.max([self._bbox[1][1], y], axis=0)
+                    y_min = np.nanmin([self._bbox[1][0], y], axis=0)
+                    y_max = np.nanmax([self._bbox[1][1], y], axis=0)
                 except ValueError:
                     # Happens when `_bbox[1]` is a float and `y` a vector.
                     y_min = y_max = y
@@ -420,7 +476,7 @@ class Learner1D(BaseLearner):
                 self._bbox[1][1] = max(self._bbox[1][1], y)
                 self._scale[1] = self._bbox[1][1] - self._bbox[1][0]
 
-    def tell(self, x, y):
+    def tell(self, x: float, y: Union[Float, Sequence[Float], np.ndarray]) -> None:
         if x in self.data:
             # The point is already evaluated before
             return
@@ -455,7 +511,7 @@ class Learner1D(BaseLearner):
 
             self._oldscale = deepcopy(self._scale)
 
-    def tell_pending(self, x):
+    def tell_pending(self, x: float) -> None:
         if x in self.data:
             # The point is already evaluated before
             return
@@ -463,7 +519,17 @@ class Learner1D(BaseLearner):
         self._update_neighbors(x, self.neighbors_combined)
         self._update_losses(x, real=False)
 
-    def tell_many(self, xs, ys, *, force=False):
+    def tell_many(
+        self,
+        xs: Sequence[Float],
+        ys: Union[
+            Sequence[Float],
+            Sequence[Sequence[Float]],
+            Sequence[np.ndarray],
+        ],
+        *,
+        force: bool = False
+    ) -> None:
         if not force and not (len(xs) > 0.5 * len(self.data) and len(xs) > 2):
             # Only run this more efficient method if there are
             # at least 2 points and the amount of points added are
@@ -483,8 +549,8 @@ class Learner1D(BaseLearner):
         points_combined = np.hstack([points_pending, points])
 
         # Generate neighbors
-        self.neighbors = _get_neighbors_from_list(points)
-        self.neighbors_combined = _get_neighbors_from_list(points_combined)
+        self.neighbors = _get_neighbors_from_array(points)
+        self.neighbors_combined = _get_neighbors_from_array(points_combined)
 
         # Update scale
         self._bbox[0] = [points_combined.min(), points_combined.max()]
@@ -494,10 +560,10 @@ class Learner1D(BaseLearner):
         self._oldscale = deepcopy(self._scale)
 
         # Find the intervals for which the losses should be calculated.
-        intervals, intervals_combined = [
+        intervals, intervals_combined = (
             [(x_m, x_r) for x_m, (x_l, x_r) in neighbors.items()][:-1]
             for neighbors in (self.neighbors, self.neighbors_combined)
-        ]
+        )
 
         # The the losses for the "real" intervals.
         self.losses = loss_manager(self._scale[0])
@@ -531,7 +597,7 @@ class Learner1D(BaseLearner):
                 # have an inf loss.
                 self._update_interpolated_loss_in_interval(*ival)
 
-    def ask(self, n, tell_pending=True):
+    def ask(self, n: int, tell_pending: bool = True) -> Tuple[List[float], List[float]]:
         """Return 'n' points that are expected to maximally reduce the loss."""
         points, loss_improvements = self._ask_points_without_adding(n)
 
@@ -541,7 +607,16 @@ class Learner1D(BaseLearner):
 
         return points, loss_improvements
 
-    def _ask_points_without_adding(self, n):
+    def _missing_bounds(self) -> List[Real]:
+        missing_bounds = []
+        for b in copy(self.__missing_bounds):
+            if b in self.data:
+                self.__missing_bounds.remove(b)
+            elif b not in self.pending_points:
+                missing_bounds.append(b)
+        return sorted(missing_bounds)
+
+    def _ask_points_without_adding(self, n: int) -> Tuple[List[float], List[float]]:
         """Return 'n' points that are expected to maximally reduce the loss.
         Without altering the state of the learner"""
         # Find out how to divide the n points over the intervals
@@ -556,19 +631,15 @@ class Learner1D(BaseLearner):
             return [], []
 
         # If the bounds have not been chosen yet, we choose them first.
-        missing_bounds = [
-            b
-            for b in self.bounds
-            if b not in self.data and b not in self.pending_points
-        ]
-
+        missing_bounds = self._missing_bounds()
         if len(missing_bounds) >= n:
             return missing_bounds[:n], [np.inf] * n
 
         # Add bound intervals to quals if bounds were missing.
         if len(self.data) + len(self.pending_points) == 0:
             # We don't have any points, so return a linspace with 'n' points.
-            return np.linspace(*self.bounds, n).tolist(), [np.inf] * n
+            a, b = self.bounds
+            return np.linspace(a, b, n).tolist(), [np.inf] * n
 
         quals = loss_manager(self._scale[0])
         if len(missing_bounds) > 0:
@@ -619,11 +690,13 @@ class Learner1D(BaseLearner):
 
         return points, loss_improvements
 
-    def _loss(self, mapping, ival):
+    def _loss(
+        self, mapping: Dict[Interval, float], ival: Interval
+    ) -> Tuple[float, Interval]:
         loss = mapping[ival]
         return finite_loss(ival, loss, self._scale[0])
 
-    def plot(self, *, scatter_or_line="scatter"):
+    def plot(self, *, scatter_or_line: str = "scatter"):
         """Returns a plot of the evaluated data.
 
         Parameters
@@ -658,22 +731,23 @@ class Learner1D(BaseLearner):
 
         return p.redim(x=dict(range=plot_bounds))
 
-    def remove_unfinished(self):
+    def remove_unfinished(self) -> None:
         self.pending_points = set()
         self.losses_combined = deepcopy(self.losses)
         self.neighbors_combined = deepcopy(self.neighbors)
 
-    def _get_data(self):
+    def _get_data(self) -> Dict[float, float]:
         return self.data
 
-    def _set_data(self, data):
+    def _set_data(self, data: Dict[float, float]) -> None:
         if data:
-            self.tell_many(*zip(*data.items()))
+            xs, ys = zip(*data.items())
+            self.tell_many(xs, ys)
 
     def __getstate__(self):
         return (
             cloudpickle.dumps(self.function),
-            self.bounds,
+            tuple(self.bounds),
             self.loss_per_interval,
             dict(self.losses),  # SortedDict cannot be pickled
             dict(self.losses_combined),  # ItemSortedDict cannot be pickled
@@ -689,17 +763,17 @@ class Learner1D(BaseLearner):
         self.losses_combined.update(losses_combined)
 
 
-def loss_manager(x_scale):
+def loss_manager(x_scale: float) -> Dict[Interval, float]:
     def sort_key(ival, loss):
         loss, ival = finite_loss(ival, loss, x_scale)
         return -loss, ival
 
-    sorted_dict = sortedcollections.ItemSortedDict(sort_key)
+    sorted_dict = ItemSortedDict(sort_key)
     return sorted_dict
 
 
-def finite_loss(ival, loss, x_scale):
-    """Get the socalled finite_loss of an interval in order to be able to
+def finite_loss(ival: Interval, loss: float, x_scale: float) -> Tuple[float, Interval]:
+    """Get the so-called finite_loss of an interval in order to be able to
     sort intervals that have infinite loss."""
     # If the loss is infinite we return the
     # distance between the two points.
