@@ -3,6 +3,7 @@ import itertools
 import random
 from collections import OrderedDict
 from collections.abc import Iterable
+from copy import deepcopy
 
 import numpy as np
 import scipy.spatial
@@ -319,6 +320,7 @@ class LearnerND(BaseLearner):
         else:
             self._bounds_points = sorted(list(map(tuple, itertools.product(*bounds))))
             self._bbox = tuple(tuple(map(float, b)) for b in bounds)
+            self._interior = None
 
         self.ndim = len(self._bbox)
 
@@ -337,6 +339,7 @@ class LearnerND(BaseLearner):
         # for the output
         self._min_value = None
         self._max_value = None
+        self._old_scale = None
         self._output_multiplier = (
             1  # If we do not know anything, do not scale the values
         )
@@ -379,6 +382,12 @@ class LearnerND(BaseLearner):
                 self._vdim = 1
         return self._vdim if self._vdim is not None else 1
 
+    def to_numpy(self):
+        """Data as NumPy array of size ``(npoints, dim+vdim)``, where ``dim`` is the
+        size of the input dimension and ``vdim`` is the length of the return value
+        of ``learner.function``."""
+        return np.array([(*p, *np.atleast_1d(v)) for p, v in sorted(self.data.items())])
+
     @property
     def bounds_are_done(self):
         return all(p in self.data for p in self._bounds_points)
@@ -447,7 +456,7 @@ class LearnerND(BaseLearner):
 
     def inside_bounds(self, point):
         """Check whether a point is inside the bounds."""
-        if hasattr(self, "_interior"):
+        if self._interior is not None:
             return self._interior.find_simplex(point, tol=1e-8) >= 0
         else:
             eps = 1e-8
@@ -900,7 +909,7 @@ class LearnerND(BaseLearner):
         else:
             raise ValueError("Only 1 or 2-dimensional plots can be generated.")
 
-    def plot_3D(self, with_triangulation=False):
+    def plot_3D(self, with_triangulation=False, return_fig=False):
         """Plot the learner's data in 3D using plotly.
 
         Does *not* work with the
@@ -910,6 +919,9 @@ class LearnerND(BaseLearner):
         ----------
         with_triangulation : bool, default: False
             Add the verticices to the plot.
+        return_fig : bool, default: False
+            Return the `plotly.graph_objs.Figure` object instead of showing
+            the rendered plot (default).
 
         Returns
         -------
@@ -980,14 +992,7 @@ class LearnerND(BaseLearner):
 
         fig = plotly.graph_objs.Figure(data=plots, layout=layout)
 
-        return plotly.offline.iplot(fig)
-
-    def _get_data(self):
-        return self.data
-
-    def _set_data(self, data):
-        if data:
-            self.tell_many(*zip(*data.items()))
+        return fig if return_fig else plotly.offline.iplot(fig)
 
     def _get_iso(self, level=0.0, which="surface"):
         if which == "surface":
@@ -1176,3 +1181,10 @@ class LearnerND(BaseLearner):
             opacity=opacity,
             lighting=lighting,
         )
+
+    def _get_data(self):
+        return deepcopy(self.__dict__)
+
+    def _set_data(self, state):
+        for k, v in state.items():
+            setattr(self, k, v)
