@@ -159,7 +159,7 @@ def live_plot(runner, *, plotter=None, update_interval=2, name=None, normalize=T
                 await asyncio.sleep(update_interval)
             event()  # fire off one last update before we die
         finally:
-            if active_plotting_tasks[name] is asyncio.Task.current_task():
+            if active_plotting_tasks[name] is asyncio.current_task():
                 active_plotting_tasks.pop(name, None)
             cancel_button.layout.display = "none"  # remove cancel button
 
@@ -251,7 +251,7 @@ def _info_html(runner):
 
     overhead = runner.overhead()
     red_level = max(0, min(int(255 * overhead / 100), 255))
-    overhead_color = "#{:02x}{:02x}{:02x}".format(red_level, 255 - red_level, 0)
+    overhead_color = f"#{red_level:02x}{255 - red_level:02x}{0:02x}"
 
     info = [
         ("status", f'<font color="{color}">{status}</font>'),
@@ -262,6 +262,9 @@ def _info_html(runner):
     with suppress(Exception):
         info.append(("# of points", runner.learner.npoints))
 
+    with suppress(Exception):
+        info.append(("# of samples", runner.learner.nsamples))
+
     with suppress(Exception):
         info.append(("latest loss", f'{runner.learner._cache["loss"]:.3f}'))
 
